/***************************************************************************
                          Crossfire.cpp  -  description
                             -------------------
    begin                : Sun May 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <spritelib/spritelib.h>
#include "Crossfire.h"
#include "Hero.h"
#include "Monster.h"
#include "JollyRoger.h"
#include "PauseSprite.h"


class Music Crossfire::music=Music::GetObject();

const char *gSound_Names[]          ={  "pfire.wav","efire.wav",
                                        "explode5.wav","explode.wav",
                                        "boom45.wav","nuke2.wav",
                                        "1up.wav","evillaugh.wav",
                                        "pause.wav",NULL                };

const char *gSprite_Names[]         ={  "pfire.spr","efire.spr","hero.spr",
                                        "monster.spr","blowup.spr",
                                        "block.spr","jolly.spr",
                                        "nuke.spr","1up.spr",
                                        "gameover.spr","cross_logo.spr",
                                        "background.spr","pause.spr",
                                        "font.spr",
                                        NULL                              };



Crossfire::Crossfire():SpriteManager(0,NULL,NULL),Sprite_Names(gSprite_Names),
    Sound_Names(gSound_Names),score(POINTS_FOR_FREELIFE,0,0,3)
{
  int x=-1,y=-1;
  mixer=NULL;
  hero=NULL;
  shots=MAX_PLAYER_SHOTS;
  monsterdeaths=0;
  gameover=0;
  Sprite_Loaders=NULL;
  Sound_Data=NULL;  
  
  srand((int)time(NULL));  /* Initialize random number generator */
  
  timer=SDL_AddTimer(BONUS_DELAY,TCallback,this);
  if(timer==NULL)
    fprintf(stderr, "Warning - couldn't create timer\n");

  if(!Init_Data())
    fprintf(stderr,"Warning - couldn't load all data\n");

  score.Init(Sprite_Loaders[GAME_SPRITE_FONT]->GetSurface(),6,6);                              
  score.MoveTo(TO_SFXP(320)-TO_SFXP(50),TO_SFXP(10));


  if(bg==NULL)
  {
    if(Sprite_Loaders==NULL)
    {
      fprintf(stderr,"Error!  Graphics not loaded\n");
      abort();
    }
    else
    {
      flag&=~SM_OWNBG;
      bg=Sprite_Loaders[GAME_SPRITE_BACKGROUND]->GetSurface();
      if(bg==NULL)
      {
        fprintf(stderr,"Error!  Background not loaded\n");
        abort();
      }
    }
  }
  else
  {
    if(Sprite_Loaders==NULL)
    {
      fprintf(stderr,"Error!  Graphics not loaded\n");
      abort();
    }
    else
    {
      VCS_Surface *nbg=Sprite_Loaders[GAME_SPRITE_BACKGROUND]->GetSurface();
      
      if(nbg==NULL)
        fprintf(stderr,"Warning!  Background not loaded\n");
      else
        VCS_BlitSurface(nbg,NULL,bg,NULL);
    }    
  }  

  fprintf(stderr,"Crossfire() Data initialized\n");

  mixer=new Mixer();

  calc_monster_pos();

  /* Draw the blocks on the background */
  for(y=1; y<(GCOUNT_Y-1); y+=2)
  for(x=1; x<(GCOUNT_X-1); x+=2)
  {
    int xpos=FROM_SFXP(G_XPos(x)),ypos=FROM_SFXP(G_YPos(y));
    VCS_Surface *surf=Sprite_Loaders[GAME_SPRITE_BLOCK]->GetSurface();
    Rect r(xpos,ypos,0,0);
    DrawOnBG(surf,NULL,&r);
  }

  SDL_ShowCursor(SDL_DISABLE);
  if(SDL_ShowCursor(SDL_QUERY)==SDL_ENABLE)
  {
    fprintf(stderr,"Warning, couldn't hide mouse pointer.\n");
  }

  Set();
}

void Crossfire::Add_Monsters()
{
  int n=-1;
  /* Add the monsters to the game */
  for(n=0; n<MAX_MONSTERS; n++)
  {
    Sprite *s=Sprite_Loaders[GAME_SPRITE_MONSTER]->GetSprite(new Monster(n,hero));
    AddSprite(s);
  }  
}

bool Crossfire::Good()
{
  int n=-1;

  if(screen==NULL)  return(false);
  else if(bg==NULL) return(false);
  
  for(n=0; n<GAME_SPRITE_LAST; n++)
  {
    if(Sprite_Loaders[n]==NULL)
      return(false);
    else if(!(Sprite_Loaders[n]->Good()))
      return(false);      
  }

  for(n=0; n<GAME_SOUND_LAST; n++)
  {
    if(Sound_Data[n]==NULL)
      return(false);
    else if(!(Sound_Data[n]->Good()))
      return(false);
  }

  return(true);    
}

void Crossfire::Reset()
{
  Clear();
  Set();
}

void Crossfire::Set()
{
  if(!Good())
  {
    fprintf(stderr,"Can't do Crossfire::Set(), not all data loaded!\n");
    return;
  }
  
  shots=MAX_PLAYER_SHOTS;
  monsterdeaths=0;
  gameover=0;

  if(hero==NULL)
  {
    hero=(Hero *)Sprite_Loaders[GAME_SPRITE_HERO]->GetSprite(new Hero());
    AddSprite(hero);
  }

  score.OnChange();
  AddSprite(&score);  

  Add_Monsters();  

  flag|=SM_TOTALREDRAW;
}

void Crossfire::Clear()
{
  if(!Good())
  {
    fprintf(stderr,"Can't do Crossfire::Clear(), not all data loaded!\n");
    return;
  }
  
  gameover=0;
  score.Reset();
  sprites.Destroy();
  if(hero!=NULL)
  {
    delete hero;
    hero=NULL;
  }  
}

void Crossfire::Free_Data()
{
  Sprite_Loaders=FreeList<SpriteLoader>(Sprite_Names,Sprite_Loaders);
  Sound_Data=FreeList<Sample>(Sound_Names,Sound_Data);
}


int Crossfire::Init_Data()
{
  int retval=1;
  Process_Conf();

  Sprite_Loaders=CreateList<SpriteLoader>(Sprite_Names,Path_Graph);
  if(Sprite_Loaders==NULL)
  {
    fprintf(stderr,"Couldn't load game-sprites!\n");
    retval=0;
  }

  Sound_Data=CreateList<Sample>(Sound_Names,Path_Sound);
  if(Sound_Data==NULL)
  {
    fprintf(stderr,"Couldn't load sounds!\n");
    retval=0;
    if(Sprite_Loaders!=NULL)
    {
      Sprite_Loaders=FreeList<SpriteLoader>(Sprite_Names,Sprite_Loaders);
    }
  }
  
  return(retval); /* Return success */
}


void Crossfire::DropNuke()
{
  int m=-1,n=-1;
  Sfxp x=-1,y=-1;
  PlaySFX(GAME_SOUND_NUKE,0,0);

  for(m=0; m<10; m++)
  {
    x=TO_SFXP(RandRange(FROM_SFXP(GOFF_X),FROM_SFXP(GMAX_X)));
    y=TO_SFXP(RandRange(FROM_SFXP(GOFF_Y),FROM_SFXP(GMAX_Y)));
    for(n=0; n<3; n++)
    {
      SfxpCoord coord;
      Sprite *spr=Sprite_Loaders[GAME_SPRITE_BLOWUP]->GetSprite(new Animation());
      spr->MoveTo(x,y);

      coord.Vector(RandRange(10,500),rand()%AFXP_MAX);
      spr->SetVel(coord);
      AddSprite(spr);
    }
  }
}
                                        
                                        
Uint32 Crossfire::OnTimer(Uint32 interval)
{
  Bonus *bonus=NULL;
  if( (!Paused()) && (flag&SM_RUNNING) )
    switch(rand()%3)
    {
    case 0:
      bonus=(Bonus *)Sprite_Loaders[GAME_SPRITE_JOLLY]->GetSprite(new JollyRoger());
      break;
      
    case 1:
      bonus=(Bonus *)Sprite_Loaders[GAME_SPRITE_NUKE]->GetSprite(new Nuke());
      break;

    case 2:
      bonus=(Bonus *)Sprite_Loaders[GAME_SPRITE_FREELIFE]->GetSprite(new FreeLife());
      break;

    default:
      break;
    }


  if(bonus!=NULL)
  {
    while(bonus->CanCollide(hero))
    {
      bonus->RandPos();
    }

    AddSprite(bonus);
  }
  
  return(BONUS_DELAY);
}

void Crossfire::Handler(SDL_Event *event)
{  
  switch(event->type)
  {
  int keystate;  
  case SDL_KEYDOWN:
    keystate=SDL_GetModState();

    switch(event->key.keysym.sym)
    {
    case SDLK_COMMA:
    case SDLK_PERIOD:
      music.Stop();
      break;
      
    case SDLK_PAUSE:
    case SDLK_F1:
      if(!gameover)
      {
        PlaySFX(GAME_SOUND_PAUSE,0,180);
        if(Paused())
        {
          Unpause();          
          PushUserEvent(GAME_EVENT_UNPAUSE);
        }
        else
        {
          PauseSprite *pause;
          pause=(PauseSprite *)Sprite_Loaders[GAME_SPRITE_PAUSE]->GetSprite(new PauseSprite());
          pause->AlignCenter();
          AddSprite(pause);       
                    
          Pause();
        }
      }
      else
      {
        Unpause();
      }
            
      break;
      
    case SDLK_RETURN:
    case SDLK_KP_ENTER:
      if(keystate&(KMOD_LALT|KMOD_RALT))
      {
        ToggleFullscreen();
        Redraw();
      }
      break;

    case SDLK_ESCAPE:
      if(!gameover)
      {
        SDL_Event nevent;
        nevent.type=SDL_QUIT;
        SDL_PushEvent(&nevent);
        Unpause();
      }

      break;

    case SDLK_LEFT:
    case SDLK_KP4:
      if(keystate&(KMOD_LCTRL|KMOD_RCTRL))
        PlayerFire(DIR_LEFT);
      else
        hero->Left();
      break;

    case SDLK_RIGHT:
    case SDLK_KP6:
      if(keystate&(KMOD_LCTRL|KMOD_RCTRL))
        PlayerFire(DIR_RIGHT);
      else
        hero->Right();
      break;

    case SDLK_UP:
    case SDLK_KP8:
      if(keystate&(KMOD_LCTRL|KMOD_RCTRL))
        PlayerFire(DIR_UP);
      else
        hero->Up();
      break;

    case SDLK_DOWN:
    case SDLK_KP2:
      if(keystate&(KMOD_LCTRL|KMOD_RCTRL))
        PlayerFire(DIR_DOWN);
      else
        hero->Down();

    default:
      break;
    }
    break;
  }
}

void Crossfire::PlayerFire(int dir)
{
  int curdir=hero->Direction();

  if(Paused()||gameover)  return;

  if(curdir)
  switch(dir)
  {
  case DIR_LEFT:
    if(curdir&(DIR_UP|DIR_DOWN))    return;
    break;

  case DIR_RIGHT:
    if(curdir&(DIR_UP|DIR_DOWN))    return;
    break;

  case DIR_UP:
    if(curdir&(DIR_LEFT|DIR_RIGHT)) return;
    break;

  case DIR_DOWN:
    if(curdir&(DIR_LEFT|DIR_RIGHT)) return;
    break;

  default:
    return;
  }

  Lock();

  if(shots<=0)
  {
    Unlock();
    return;
  }

  Unlock();
  

  Shot *shot=(Shot *)Sprite_Loaders[GAME_SPRITE_PFIRE]->GetSprite(new PShot());
  shot->Start(hero,250,dir);
  AddSprite(shot);
  PlaySFX(GAME_SOUND_PFIRE,PLAYER_SHOT_DISTANCE,G_CalcPan(hero->Xfxp()));
}


void Crossfire::EnemyFire(GridSprite *spr, int dir)
{
  Monster *m=(Monster *)spr;
  if((dir!=0)&&(m->Mode()!=BHV_HIDE))
  {
    int pan=G_CalcPan(spr->Xfxp());
    Shot *shot=(Shot *)Sprite_Loaders[GAME_SPRITE_EFIRE]->GetSprite(new Shot());
    shot->Start(spr,MONSTER_SHOT_SPEED,dir);
    AddSprite(shot);
    PlaySFX(GAME_SOUND_EFIRE,MIX_MAX_VOLUME/2,pan);
  }
}


int Crossfire::PlaySFX(int index, Uint8 dist, Sint16 pan)
{  
  if(index<0) return(0);  
  else if(index>=GAME_SOUND_LAST) return(0);
  else if(!Sound_Data[index]->Good()) return(0);  
  else return(mixer->Play(Sound_Data[index][0],dist,pan));  
}

void Crossfire::Idle()
{
  if(Mus_Names.Size()!=0)
  if(!music.Playing())
  {
    int song=rand()%Mus_Names.Size();
    if(!music.Play(Mus_Names[song]))
    {
      fprintf(stderr,"Warning, couldn't play %s\n",Mus_Names[song]);
      fprintf(stderr,"File may be absent or invalid\n");
    }
  }
}

int Crossfire::Event(int code, void *data1, void *data2)
{
  GameOver *over=NULL;
  
  switch(code)
  {
  case GAME_EVENT_PSHOTSTART:
    Lock();
    shots--;
    Unlock();
    return(1);
    
  case GAME_EVENT_GAMEOVER:
    fprintf(stderr,"Game over.  Points: %d High: %d\n",score.Points(),score.High());
    gameover=1;
    over=(GameOver *)Sprite_Loaders[GAME_SPRITE_GAMEOVER]->GetSprite(new GameOver());
    over->AlignCenter();
    AddSprite(over);
    Unpause();
    return(0);

  case GAME_EVENT_MONSTERGONE:
    Lock();
    monsterdeaths++;
    if(monsterdeaths==(MAX_MONSTERS-DEATHHUNT_LEVEL))
    {
      PushUserEvent(GAME_EVENT_DEATHHUNT);
    }
    else if(monsterdeaths==MAX_MONSTERS)
    {
      fprintf(stderr,"New level!\n");
      Add_Monsters();
      monsterdeaths=0;
    }
    Unlock();    
    return(1);  
    
  case GAME_EVENT_MONSTERDIED:
    ScorePoints(POINTS_MONSTER);
    return(1);
    
  case GAME_EVENT_NUKE:
    ScorePoints(POINTS_NUKE);
    DropNuke();  
    return(0);
    
  case GAME_EVENT_FREELIFE:
    ScorePoints(POINTS_LIFE);
    PlaySFX(GAME_SOUND_FREELIFE,0,0);
    Lock();
    score.AddLives();

    Unlock();
    return(1);
    break;
    
  case GAME_EVENT_PLAYERDIED:
    Lock();
    score.Die();
    Unlock();
    
    if(score.Lives()==0)
    {
      PushUserEvent(GAME_EVENT_GAMEOVER);
      return(1);
    }
    else
    {
      return(0);
    }

    
  case GAME_EVENT_EXPLOSION:
    Explosion((Sfxp)data1,(Sfxp)data2);
    return(1);
    
  case GAME_EVENT_PSHOTDONE:
    Lock();
    shots++;
    Unlock();
    return(1);

  case GAME_EVENT_EFIRE:
    EnemyFire((GridSprite *)data1,(int)data2);
    return(1);

  case GAME_EVENT_PLAYSFX:
    if(data2==NULL)
    {
      PlaySFX((int)data1,64,0);
    }
    else
    {
      GridSprite *spr=(GridSprite *)data2;
      int pan=G_CalcPan(spr->Xfxp());
      PlaySFX((int)data1,64,pan);
    }
    return(1);
    
  default:
    return(0);
  }

  return(0);
}

void Crossfire::Explosion(Sfxp x, Sfxp y)
{  
  int n;

  int SoundOdds[]={1,1,1,-1};
  int Sound[]={GAME_SOUND_EXPLODE,GAME_SOUND_EXPLODE2,GAME_SOUND_EXPLODE3};

  PlaySFX(SwitchOdds(SoundOdds,Sound),MIX_MAX_VOLUME/2,G_CalcPan(x) );

  for(n=0; n<3; n++)
  {
    SfxpCoord coord;
    Sprite *spr=Sprite_Loaders[GAME_SPRITE_BLOWUP]->GetSprite(new Animation());
    spr->MoveTo(x,y);

    coord.Vector(RandRange(10,500),rand()%AFXP_MAX);
    spr->SetVel(coord);
    AddSprite(spr);
  }
  
}

void Crossfire::ScorePoints(int amount)
{
  Lock();

  if(score.Add(amount))
  {
    PushUserEvent(GAME_EVENT_FREELIFE);
  }

  Unlock();
}



Crossfire::~Crossfire()
{
  if(timer!=NULL)
  { 
    SDL_RemoveTimer(timer);
  }
  else
  {
    fprintf(stderr,"~Crossfire(): Can't remove NULL timer\n");
  }  
  
  if(mixer!=NULL) /* Stop sound */
  {
    mixer->Stop();
    delete mixer;
    mixer=NULL;
  }
  else
  {
    fprintf(stderr,"~Crossfire():  Can't delete NULL mixer\n");
  }

  Clear();  /* Clear the game board */
  
  Free_Data(); /* Unload all game data */
  
  fprintf(stderr,"~Crossfire():  Data freed\n");
}



