/***************************************************************************
                          CrossfireConf.cpp  -  description
                             -------------------
    begin                : Wed May 28 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Crossfire.h"
#include <spritelib/toyparse.h>

#define CFILE_NAME ".crossfire"
#define MFILE_NAME "cross_mus"

#ifdef _WIN32_WCE
#define DEFAULT_PATH "\\crossdata"
#elif defined(WIN32)
#define DEFAULT_PATH ".\\crossdata"
#else
#define DEFAULT_PATH "/usr/share/crossfire"
#endif

const char *Default_Param[] =
{
  "\"" DEFAULT_PATH SLASH "graph" SLASH,
  "\"" DEFAULT_PATH SLASH "sound" SLASH,
  "\"" DEFAULT_PATH SLASH MFILE_NAME,
  NULL
};

/*! \brief Recognizes valid parameter types */
const char *CType_Param[]  ={ "GRAPH_PATH","SOUND_PATH", "MUSIC_LIST",
                              NULL };
/*! \brief Recognizes comments */
const char *CType_Comment[]={ "#", "#%s", NULL};
/*! \brief Recognizes integers */
const char *CType_String[] ={  "\"%s", NULL    };
/*! \brief Complete command table */
const char **CTypes[]      ={  CType_Comment  ,
                              CType_Param    ,
                              CType_String   ,
                              NULL            };                              
enum CrossfireConfType
{
  CTYPE_COMMENT=0,
  CTYPE_PARAM,
  CTYPE_STRING,
  CTYPE_ERROR=-1
};

enum CrossfireParamType
{
  CPARAM_GPATH=0,
  CPARAM_SPATH,
  CPARAM_MLIST,
  CPARAM_ERROR=-1
};

FILE *GetCrossfireConf();

void ParseFileList(StringTable &path, StringTable &files, FILE *fp)
{
  char *sep=SLASH;
  char buf[300]={'\0'};
  if(fp==NULL) return;

  if(fgets(buf,299,fp)!=NULL)
  while(!feof(fp))
  {
    int len=0;
    StripNewline(buf);

    len=strlen(buf);
        
    if(buf[len-1]==sep[0])
    {
      path.Add(buf);
    }
    else
    {
      files.Add(buf);
    }
            
    if(fgets(buf,299,fp)==NULL) break;
  }
}

void Crossfire::Process_Conf()
{
  int line=1;
  char buf[300]={'\0'};
  char file_music_list[300]={'\0'};
  
  FILE *fp=GetCrossfireConf();
  if(fp==NULL)
  {
    fprintf(stderr,"Couldn't find "CFILE_NAME"!\n");
    return;
  }

  if(fgets(buf,299,fp)!=NULL)
  while(!feof(fp))
  {
    TokenType *types=NULL;
        
    types=LineToTypes(buf,CTypes);

    if(types!=NULL)
    {
      switch(types[0].type)
      {
      case CTYPE_COMMENT:      
        break;
        
      case CTYPE_PARAM:
        if(CheckLineSyntax(types,CTYPE_PARAM,-1,CTYPE_STRING,-1,TYPE_ENDCLOSE)<0)
        {
          fprintf(stderr,".crossfire[%2d]: Invalid syntax\n",line);
          fprintf(stderr,"%s",buf);
          break;          
        }
      
        switch(types[0].subtype)
        {
        case CPARAM_GPATH:
          strcpy(Path_Graph,types[1].token+1);
          break;

        case CPARAM_SPATH:
          strcpy(Path_Sound,types[1].token+1);        
          break;

        case CPARAM_MLIST:
          strcpy(file_music_list,types[1].token+1);
          break;

        default:        
          break;
        }        
        break;
      case CTYPE_STRING:
        break;
      default:
        fprintf(stderr,".crossfire[%2d]: Invalid syntax\n",line);
        fprintf(stderr,"%s",buf);
        break;
      }
      
      FreeType(types);
      line++;
      if(fgets(buf,300,fp)==NULL) break;      
    }    
  }
  
  fclose(fp);

  if(!Mus_Names.LoadFile(file_music_list))
  {
    fprintf(stderr,"Couldn't load music list %s\n",file_music_list);
  }
}


FILE *CreateCrossfireConf()
{
  char buf[300]={'\0'};
  char *home=NULL;
  int n=-1;

  fprintf(stderr,"Unable to find %s, so creating a new one..\n",CFILE_NAME);  

  home=getenv("HOME");
  if(home!=NULL)
  {
    strcpy(buf,home);
    strcat(buf,SLASH CFILE_NAME);    
  }
  else
  { strcpy(buf, "." SLASH CFILE_NAME);  }  

  fprintf(stderr,"Creating %s\n",buf);    
  FILE *fp=fopen(buf,"w");  

  if(fp==NULL)
  {
    fprintf(stderr,"Unable to create %s\n",buf);
    return(NULL);
  }

  for(n=0; CType_Param[n]!=NULL; n++)
  {
    if(fprintf(fp,"%s  %s\n",CType_Param[n],Default_Param[n])<0)
    {
      fprintf(stderr,"Error writing to %s!\n",buf);
    }
  }
  
  fclose(fp);

  fp=fopen(buf,"r");
  
  if(fp==NULL)
  {
    fprintf(stderr,"Error reopening new conf file\n");
  }
  
  return(fp);
}

FILE *GetCrossfireConf()
{
  FILE *fp=NULL;

  fp=fopen("."SLASH CFILE_NAME,"r");
  if(fp!=NULL) fprintf(stderr,"Opened %s\n","."SLASH CFILE_NAME);
    
  if(fp==NULL)  fp=fopen(CFILE_NAME,"r");
  if(fp!=NULL) fprintf(stderr,"Opened %s\n",CFILE_NAME);  

  
  if(fp==NULL)
  {
    char buf[300]={'\0'};
    char *home=getenv("HOME");
    if(home!=NULL)
    {
      strcpy(buf,home);
      strcat(buf,SLASH CFILE_NAME);
      fp=fopen(buf,"r");
      if(fp!=NULL) fprintf(stderr,"Opened %s\n",buf);
    }
  }
  
  if(fp==NULL)
  {
    char buf[300]={'\0'};    
    strcpy(buf,SLASH "etc" SLASH CFILE_NAME);
    fp=fopen(buf,"r");
    if(fp!=NULL) fprintf(stderr,"Opened %s\n",buf);    
  }

  if(fp==NULL)  /* If it STILL can't find it, make new one in home dir */
  {
    fp=CreateCrossfireConf();
  }

  return(fp);
}

