/***************************************************************************
                          Hero.cpp  -  description
                             -------------------
    begin                : Fri Jun 6 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Hero.h"

/**
 *  Ensures that the player's sprite doesn't get deleted when it
 *  expires, so the object still exists after it's taken off the board
 */
Hero::Hero():GridSprite(HERO_START_GRIDX,HERO_START_GRIDY,HERO_SPEED)
{
  ClearFlags(SPRITE_DELETEABLE);
}

/**
 *  Produces a GAME_EVENT_PLAYERDIED event, causes an explosion at the
 *  player's coordinates, and resets the sprite to it's home coordinates.
 */
void Hero::Die()
{
  PushUserEvent(GAME_EVENT_PLAYERDIED);
  PushUserEvent(GAME_EVENT_EXPLOSION,(void *)Xfxp(),(void *)Yfxp());
  Reset();
}

/**
 *  Removes the sprite from the game board when a
 *  GAME_EVENT_GAMEOVER event is recieved.
 */
void Hero::Event(int code, void *data1, void *data2)
{
  switch(code)
  {
  case GAME_EVENT_GAMEOVER:
    Expire();
    break;

  default:
    break;
  }
}

/**
 *  Kills the player sprite on collision with enemies,
 *  enemy bullets, and the Jolly Roger.
 */
void Hero::OnCollide(const Sprite *spr, ColType index)
{
  switch(index)
  {
  case GAME_LAYER_JOLLY:
  case GAME_LAYER_EFIRE:
  case GAME_LAYER_ENEMY:
    Die();
    break;

  default:
    break;
  }
}

/**
 *  Moves the player sprite to it's home coordinates.
 */
void Hero::Reset()
{
  gridx=HERO_START_GRIDX;
  gridy=HERO_START_GRIDY;
  SnapTo(gridx,gridy);
}

void Hero::Up()
{
  int dir=Direction();
  if((dir==0)||(dir==DIR_DOWN))
  {
    gridy=LimitY(gridy-2);
  }
}

void Hero::Down()
{
  int dir=Direction();
  if((dir==0)||(dir==DIR_UP))
  {
    gridy=LimitY(gridy+2);
  }
}


void Hero::Left()
{
  int dir=Direction();
  if((dir==0)||(dir==DIR_RIGHT))
  {
    gridx=LimitX(gridx-2);
  }
}


void Hero::Right()
{
  int dir=Direction();
  if((dir==0)||(dir==DIR_LEFT))
  {
    gridx=LimitX(gridx+2);
  }
}
