/***************************************************************************
                          Hero.h  -  description
                             -------------------
    begin                : Mon May 19 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __HERO_H__
#define __HERO_H__

#define __GAME_CPP__
#include "GridSprite.h"
#include "Crossfire.h"

/*! \brief The minimum X grid coordinate the player sprite can occupy */
#define HERO_MIN_GRIDX 2
/*! \brief The maximum X grid coordinate the player sprite can occupy */
#define HERO_MAX_GRIDX (GCOUNT_X-3)
/*! \brief The minimum Y grid coordinate the player sprite can occupy */
#define HERO_MIN_GRIDY 2
/*! \brief The maximum Y grid coordinate the player sprite can occupy */
#define HERO_MAX_GRIDY (GCOUNT_Y-3)

/*! \brief The X grid coordinate the player sprite appears at */
#define HERO_START_GRIDX 6
/*! \brief The Y grid coordinate the player sprite appears at */
#define HERO_START_GRIDY 6
/*! \brief The speed the player sprite travels */
#define HERO_SPEED 250

/*! \brief The player sprite class */
/**
 *
 */
class Hero:public GridSprite
{
public:
  /*! \brief Constructor */
  Hero();
  /*! \brief Handler for SDL_USEREVENTs */  
  virtual void Event(int code, void *data1, void *data2);
  /*! \brief Collision handler */
  virtual void OnCollide(const Sprite *spr, ColType index);
  /*! \brief Function called when the player sprite dies */  
  void Die();
  /*! \brief Resets the player sprite to it's home coordinates */
  void Reset();
  /*! \brief Moves the sprite up on the game grid */
  void Up();
  /*! \brief Moves the sprite down on the game grid */
  void Down();
  /*! \brief Moves the sprite right on the game grid */
  void Right();
  /*! \brief Moves the sprite left on the game grid */  
  void Left();

  /*! \brief Limit's the players movements to the game board */    
  inline int LimitY(int gy)
  {
    if(gy<HERO_MIN_GRIDY)       return(HERO_MIN_GRIDY);
    else if(gy>HERO_MAX_GRIDY)  return(HERO_MAX_GRIDY);
    else                        return(gy);
  }

  /*! \brief Limits the player's movements to the game board */
  inline int LimitX(int gx)
  {
    if(gx<HERO_MIN_GRIDX)       return(HERO_MIN_GRIDX);
    else if(gx>HERO_MAX_GRIDX)  return(HERO_MAX_GRIDX);
    else                        return(gx);
  }
};

#endif/*__HERO_H__*/
