/***************************************************************************
                          Score.h  -  description
                             -------------------
    begin                : Thu Jun 5 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __SCORE_H__
#define __SCORE_H__

/*! \brief A class to keep track of game scores */
class Score
{
public:
  /*! \brief Constructor, with optional starting scores parameters  */
  Score(int start_freelife, int startscore=0,
        int starthigh=0, int defaultlives_in=0  )
  {
    score=startscore;
    hiscore=starthigh;
    freelife=start_freelife;
    lives=defaultlives=defaultlives_in;
    next_freelife=start_freelife;
  }

  /*! \brief Returns the current score      */
  inline int Points() { return score; }
  /*! \brief Returns the high score         */
  inline int High()   { return hiscore; }
  /*! \brief Returns the current # of lives */
  inline int Lives()  { return lives;   }
  /*! \brief Decrements # of lives, returns 0 when out of lives */
  inline int Die()
  {
    int retval=0;
    if(lives==0)
    {
      OnChange();      
      return(0);
    }
    else
    {
      retval=--lives;
      OnChange();
      return(retval);
    }
  }

  inline void AddLives(int num=1)
  {    
    if(num<0) return;
    else
    {
      lives+=num;
      OnChange();      
    }
  }

  /*! Increments score, returns nonzero if a life is won */
  inline int Add(int amount)
  {
    if(amount<=0) return(0);  /* Negative scoring BAD! */

    score+=amount;
    if(score>hiscore) hiscore=score;

    OnChange();
    
    if(score>=next_freelife)
    {
      next_freelife+=freelife;
      OnChange();
      return(1);
    }
    else
    {
      return(0);
    }
  }

  /*! \brief Resets score to zero, keeping highsore */
  inline void Reset()
  {
    score=0;
    lives=defaultlives;
    freelife=freelife;
    OnChange();    
  }

  virtual void OnChange() { }
  
private:
  /*! \brief Current score variable                 */
  int score;
  /*! \brief High score variable                    */
  int hiscore;
  /*! \brief Current lives left                     */
  int lives;
  /*! \brief Default # of free lives                */
  int defaultlives;
  /*! \brief Score the next freelife will be won at */
  int next_freelife;
  /*! \brief The number of points per freelife      */
  int freelife;
};

#endif/*__SCORE_H__*/
