/***************************************************************************
                          StringSprite.cpp  -  description
                             -------------------
    begin                : Wed Jun 11 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "StringSprite.h"

StringSprite::StringSprite()
{
  font=NULL;
  width=lines=0;
}

StringSprite::StringSprite(SDL_Surface *font_in, int width_in, int lines_in)
{
  font=NULL;
  width=lines=0;
  Init(font_in,width_in,lines_in);
}

void StringSprite::Init(SDL_Surface *font_in, int width_in, int lines_in)
{
  width=width_in;
  lines=lines_in;
  ClearFlags(SPRITE_DELETEABLE);
  charblock.Set(0,0,0,0);
  font=NULL;

  if(font_in==NULL)
  {
    fprintf(stderr,"StringSprite::Init() Warning, NULL font!\n");
    return;
  }

  font=font_in;
  charblock.Set(0,0,font->w/16,font->h/16);
  Init_Surface(CreateSurface(width,lines),true);
  Init_Frames(1,1);
  Init_Layers(0);
  if(surface==NULL)
  {
    fprintf(stderr,"Couldn't init surface\n");
    return;
  }

  Clear();
}

void StringSprite::Clear()
{
  if(surface!=NULL)
    SDL_FillRect(surface,NULL,SDL_MapRGB(surface->format,255,0,255));
}

int StringSprite::Print(const char *cmdstr, ...)
{
  int retval=-1,n=-1;
  char buf[1024]={'\0'};
  va_list ap;
  Rect pos(0,0,charblock.w,charblock.h);

  /* Convert printf-style arguments to string */
  va_start(ap,cmdstr);
    retval=vsprintf(buf,cmdstr,ap);
  va_end(ap);

  /* Blank current contents of image */
  Clear();

  /* Loop through all characters in string  */
  for(n=0; buf[n]!='\0'; n++)
  {
    switch(buf[n])
    {
      SDL_Rect r;
      case '\n':  /* '\n' is a combination carriage-return newline */

        pos.y+=charblock.h; /* Advance to next line */
        /* Falls through to '\r'  */
      case '\r':
        pos.x=0;  /* Return to start of line  */
        break;

      default:
        r=GetRect(buf[n]);  /* Get source rect for character  */

        SDL_BlitSurface(font,&r,surface,&pos);  /* Blit onto image  */

        /* Increment position */
        pos.x+=charblock.w;
        if(pos.x>=surface->w)
        {
          pos.x=0;
          pos.y+=charblock.h;
        }
        break;
    }
  }

  return(retval);
}

SDL_Surface *StringSprite::CreateSurface(int width, int height)
{
  SDL_Surface *s1=NULL;
  SDL_Surface *screen=SDL_GetVideoSurface();

  if(screen==NULL)
  {
    fprintf(stderr,"Can't get video surface!\n");
    return(NULL);
  }

  s1=SDL_CreateRGBSurface( SDL_SWSURFACE, width*charblock.w,
                           lines*charblock.h,
                           screen->format->BitsPerPixel,
                           screen->format->Rmask,
                           screen->format->Gmask,
                           screen->format->Bmask,
                           screen->format->Amask);
  if(s1==NULL)
  {
    fprintf(stderr,"Couldn't create surface\n");
  }

  SDL_SetColorKey(s1,SDL_SRCCOLORKEY,SDL_MapRGB(s1->format,255,0,255));
  return(s1);
}
